//========================================================================
// Simple FPGA Cell
//========================================================================

`ifndef CLB_SV
`define CLB_SV

`include "fu.sv"
`include "vc/muxes.sv"

module clb
(
  (* noprune, keep = 1 *) input logic [22:0] p,

  (* noprune, keep = 1 *) input  logic        clk_in,
  (* noprune, keep = 1 *) input  logic        clr_in,

  (* noprune, keep = 1 *) input  logic        n_in,
  (* noprune, keep = 1 *) input  logic        s_in,
  (* noprune, keep = 1 *) input  logic        e_in,
  (* noprune, keep = 1 *) input  logic        w_in,

  (* noprune, keep = 1 *) input logic         n4_in,
  (* noprune, keep = 1 *) input logic         s4_in,
  (* noprune, keep = 1 *) input logic         e4_in,
  (* noprune, keep = 1 *) input logic         w4_in,

  (* noprune, keep = 1 *) output logic        n_out,
  (* noprune, keep = 1 *) output logic        s_out,
  (* noprune, keep = 1 *) output logic        e_out,
  (* noprune, keep = 1 *) output logic        w_out
);

  (* noprune, keep = 1 *) logic [1:0] n_mux_sel;
  (* noprune, keep = 1 *) logic [1:0] s_mux_sel;
  (* noprune, keep = 1 *) logic [1:0] e_mux_sel;
  (* noprune, keep = 1 *) logic [1:0] w_mux_sel;

  (* noprune, keep = 1 *) logic [2:0] x1_mux_sel;
  (* noprune, keep = 1 *) logic [2:0] x2_mux_sel;
  (* noprune, keep = 1 *) logic [2:0] x3_mux_sel;

  (* noprune, keep = 1 *) logic [5:0] fu_p;

  (* noprune, keep = 1 *) logic       x1;
  (* noprune, keep = 1 *) logic       x2;
  (* noprune, keep = 1 *) logic       x3;

  (* noprune, keep = 1 *) logic       f;

  assign n_mux_sel = p[1:0];
  assign s_mux_sel = p[3:2];
  assign e_mux_sel = p[5:4];
  assign w_mux_sel = p[7:6];

  assign x1_mux_sel = p[10:8];
  assign x2_mux_sel = p[13:11];
  assign x3_mux_sel = p[16:14];

  assign fu_p = p[22:17];

  fu fu
  (
    .clk   (clk_in),
    .reset (clr_in),
    .p     (fu_p),
    .x1    (x1),
    .x2    (x2),
    .x3    (x3),
    .f     (f)
  );

  vc_Mux8#(1) x1_mux
  (
    .sel (x1_mux_sel),
    .in0 (n_in),
    .in1 (s_in),
    .in2 (e_in),
    .in3 (w_in),
    .in4 (n4_in),
    .in5 (s4_in),
    .in6 (e4_in),
    .in7 (w4_in),
    .out (x1)
  );

  vc_Mux8#(1) x2_mux
  (
    .sel (x2_mux_sel),
    .in0 (n_in),
    .in1 (s_in),
    .in2 (e_in),
    .in3 (w_in),
    .in4 (n4_in),
    .in5 (s4_in),
    .in6 (e4_in),
    .in7 (w4_in),
    .out (x2)
  );

  vc_Mux8#(1) x3_mux
  (
    .sel (x3_mux_sel),
    .in0 (n_in),
    .in1 (s_in),
    .in2 (e_in),
    .in3 (w_in),
    .in4 (n4_in),
    .in5 (s4_in),
    .in6 (e4_in),
    .in7 (w4_in),
    .out (x3)
  );

  vc_Mux4#(1) n_mux
  (
    .sel (n_mux_sel),
    .in0 (n_in),
    .in1 (e_in),
    .in2 (w_in),
    .in3 (f),
    .out (n_out)
  );

  vc_Mux4#(1) s_mux
  (
    .sel (s_mux_sel),
    .in0 (s_in),
    .in1 (e_in),
    .in2 (w_in),
    .in3 (f),
    .out (s_out)
  );

  vc_Mux4#(1) e_mux
  (
    .sel (e_mux_sel),
    .in0 (n_in),
    .in1 (s_in),
    .in2 (e_in),
    .in3 (f),
    .out (e_out)
  );

  vc_Mux4#(1) w_mux
  (
    .sel (w_mux_sel),
    .in0 (n_in),
    .in1 (s_in),
    .in2 (w_in),
    .in3 (f),
    .out (w_out)
  );

endmodule

`endif /* CLB_SV */
