//========================================================================
// 4x4 array of FPGA cells
//========================================================================

`ifndef FPGA_SV
`define FPGA_SV

`include "block_4x4.sv"

module fpga
(
  (* noprune, keep = 1 *) input  logic [4031:0] p,

  (* noprune, keep = 1 *) input  logic       g_clr,
  (* noprune, keep = 1 *) input  logic       g_clk,

  (* noprune, keep = 1 *) input  logic       n_in    [2:0][3:0],
  (* noprune, keep = 1 *) input  logic       s_in    [2:0][3:0],
  (* noprune, keep = 1 *) input  logic       e_in    [2:0][3:0],
  (* noprune, keep = 1 *) input  logic       w_in    [2:0][3:0],

  (* noprune, keep = 1 *) output logic       n_out   [2:0][3:0],
  (* noprune, keep = 1 *) output logic       s_out   [2:0][3:0],
  (* noprune, keep = 1 *) output logic       e_out   [2:0][3:0],
  (* noprune, keep = 1 *) output logic       w_out   [2:0][3:0]
);

  (* noprune, keep = 1 *) logic clk_out   [2:0][2:0][3:0];
  (* noprune, keep = 1 *) logic clr_out   [2:0][2:0][3:0];

  (* noprune, keep = 1 *) logic n4c_out   [2:0][2:0][3:0];
  (* noprune, keep = 1 *) logic s4c_out   [2:0][2:0][3:0];
  (* noprune, keep = 1 *) logic e4c_out   [2:0][2:0][3:0];
  (* noprune, keep = 1 *) logic w4c_out   [2:0][2:0][3:0];

  (* noprune, keep = 1 *) logic pn4c_out  [2:0][2:0][3:0];
  (* noprune, keep = 1 *) logic ps4c_out  [2:0][2:0][3:0];
  (* noprune, keep = 1 *) logic pe4c_out  [2:0][2:0][3:0];
  (* noprune, keep = 1 *) logic pw4c_out  [2:0][2:0][3:0];

  (* noprune, keep = 1 *) logic nc_out    [2:0][2:0][3:0];
  (* noprune, keep = 1 *) logic sc_out    [2:0][2:0][3:0];
  (* noprune, keep = 1 *) logic ec_out    [2:0][2:0][3:0];
  (* noprune, keep = 1 *) logic wc_out    [2:0][2:0][3:0];

  genvar i, j;
  generate
    for (i=0; i<=2; i=i+1) begin : rows
      for (j=0; j<=2; j=j+1) begin : columns
        if (i==0) begin
          if (j==0) begin
            block_4x4 block
            (
              .p       (p[(i*3*448)+(j*448)+447:(i*3*448)+(j*448)]),
              .g_clr   (g_clr),
              .g_clk   (g_clk),
              .clk_in  (clk_out[i+1][j]),
              .clr_in  (clr_out[i+1][j]),
              .n4_in   (n4c_out[i+1][j]),
              .s4_in   ('{1'b0, 1'b0, 1'b0, 1'b0}),
              .e4_in   ('{1'b0, 1'b0, 1'b0, 1'b0}),
              .w4_in   (w4c_out[i][j+1]),
              .pn4_in  (pn4c_out[i+1][j]),
              .ps4_in  ('{1'b0, 1'b0, 1'b0, 1'b0}),
              .pe4_in  ('{1'b0, 1'b0, 1'b0, 1'b0}),
              .pw4_in  (pw4c_out[i][j+1]),
              .n_in    (n_in[j]),
              .s_in    (nc_out[i+1][j]),
              .e_in    (wc_out[i][j+1]),
              .w_in    (w_in[i]),
              .clk_out (clk_out[i][j]),
              .clr_out (clr_out[i][j]),
              .n4_out  (n4c_out[i][j]),
              .s4_out  (s4c_out[i][j]),
              .e4_out  (e4c_out[i][j]),
              .w4_out  (w4c_out[i][j]),
              .pn4_out (pn4c_out[i][j]),
              .ps4_out (ps4c_out[i][j]),
              .pe4_out (pe4c_out[i][j]),
              .pw4_out (pw4c_out[i][j]),
              .n_out   (n_out[j]),
              .s_out   (sc_out[i][j]),
              .e_out   (ec_out[i][j]),
              .w_out   (w_out[i])
            );
          end
          else if (j==2) begin
            block_4x4 block
            (
              .p       (p[(i*3*448)+(j*448)+447:(i*3*448)+(j*448)]),
              .g_clr   (g_clr),
              .g_clk   (g_clk),
              .clk_in  (clk_out[i+1][j]),
              .clr_in  (clr_out[i+1][j]),
              .n4_in   (n4c_out[i+1][j]),
              .s4_in   ('{1'b0, 1'b0, 1'b0, 1'b0}),
              .e4_in   (e4c_out[i][j-1]),
              .w4_in   ('{1'b0, 1'b0, 1'b0, 1'b0}),
              .pn4_in  (pn4c_out[i+1][j]),
              .ps4_in  ('{1'b0, 1'b0, 1'b0, 1'b0}),
              .pe4_in  (pe4c_out[i][j-1]),
              .pw4_in  ('{1'b0, 1'b0, 1'b0, 1'b0}),
              .n_in    (n_in[j]),
              .s_in    (nc_out[i+1][j]),
              .e_in    (e_in[i]),
              .w_in    (ec_out[i][j-1]),
              .clk_out (clk_out[i][j]),
              .clr_out (clr_out[i][j]),
              .n4_out  (n4c_out[i][j]),
              .s4_out  (s4c_out[i][j]),
              .e4_out  (e4c_out[i][j]),
              .w4_out  (w4c_out[i][j]),
              .pn4_out (pn4c_out[i][j]),
              .ps4_out (ps4c_out[i][j]),
              .pe4_out (pe4c_out[i][j]),
              .pw4_out (pw4c_out[i][j]),
              .n_out   (n_out[j]),
              .s_out   (sc_out[i][j]),
              .e_out   (e_out[i]),
              .w_out   (wc_out[i][j])
            );
          end
          else begin
            block_4x4 block
            (
              .p       (p[(i*3*448)+(j*448)+447:(i*3*448)+(j*448)]),
              .g_clr   (g_clr),
              .g_clk   (g_clk),
              .clk_in  (clk_out[i+1][j]),
              .clr_in  (clr_out[i+1][j]),
              .n4_in   (n4c_out[i+1][j]),
              .s4_in   ('{1'b0, 1'b0, 1'b0, 1'b0}),
              .e4_in   (e4c_out[i][j-1]),
              .w4_in   (w4c_out[i][j+1]),
              .pn4_in  (pn4c_out[i+1][j]),
              .ps4_in  ('{1'b0, 1'b0, 1'b0, 1'b0}),
              .pe4_in  ('{1'b0, 1'b0, 1'b0, 1'b0}),
              .pw4_in  ('{1'b0, 1'b0, 1'b0, 1'b0}),
              .n_in    (n_in[j]),
              .s_in    (nc_out[i+1][j]),
              .e_in    (wc_out[i][j+1]),
              .w_in    (ec_out[i][j-1]),
              .clk_out (clk_out[i][j]),
              .clr_out (clr_out[i][j]),
              .n4_out  (n4c_out[i][j]),
              .s4_out  (s4c_out[i][j]),
              .e4_out  (e4c_out[i][j]),
              .w4_out  (w4c_out[i][j]),
              .pn4_out (pn4c_out[i][j]),
              .ps4_out (ps4c_out[i][j]),
              .pe4_out (pe4c_out[i][j]),
              .pw4_out (pw4c_out[i][j]),
              .n_out   (n_out[j]),
              .s_out   (sc_out[i][j]),
              .e_out   (ec_out[i][j]),
              .w_out   (wc_out[i][j])
            );
          end
        end

        if (i==1) begin
          if (j==0) begin
            block_4x4 block
            (
              .p       (p[(i*3*448)+(j*448)+447:(i*3*448)+(j*448)]),
              .g_clr   (g_clr),
              .g_clk   (g_clk),
              .clk_in  (clk_out[i+1][j]),
              .clr_in  (clr_out[i+1][j]),
              .n4_in   (n4c_out[i+1][j]),
              .s4_in   (s4c_out[i-1][j]),
              .e4_in   ('{1'b0, 1'b0, 1'b0, 1'b0}),
              .w4_in   (w4c_out[i][j+1]),
              .pn4_in  (pn4c_out[i+1][j]),
              .ps4_in  (ps4c_out[i-1][j]),
              .pe4_in  ('{1'b0, 1'b0, 1'b0, 1'b0}),
              .pw4_in  (pw4c_out[i][j+1]),
              .n_in    (sc_out[i-1][j]),
              .s_in    (nc_out[i+1][j]),
              .e_in    (wc_out[i][j+1]),
              .w_in    (w_in[i]),
              .clk_out (clk_out[i][j]),
              .clr_out (clr_out[i][j]),
              .n4_out  (n4c_out[i][j]),
              .s4_out  (s4c_out[i][j]),
              .e4_out  (e4c_out[i][j]),
              .w4_out  (w4c_out[i][j]),
              .pn4_out (pn4c_out[i][j]),
              .ps4_out (ps4c_out[i][j]),
              .pe4_out (pe4c_out[i][j]),
              .pw4_out (pw4c_out[i][j]),
              .n_out   (nc_out[i][j]),
              .s_out   (sc_out[i][j]),
              .e_out   (ec_out[i][j]),
              .w_out   (w_out[i])
            );
          end
          else if (j==2) begin
            block_4x4 block
            (
              .p       (p[(i*3*448)+(j*448)+447:(i*3*448)+(j*448)]),
              .g_clr   (g_clr),
              .g_clk   (g_clk),
              .clk_in  (clk_out[i+1][j]),
              .clr_in  (clr_out[i+1][j]),
              .n4_in   (n4c_out[i+1][j]),
              .s4_in   (s4c_out[i-1][j]),
              .e4_in   (e4c_out[i][j-1]),
              .w4_in   ('{1'b0, 1'b0, 1'b0, 1'b0}),
              .pn4_in  (pn4c_out[i+1][j]),
              .ps4_in  (ps4c_out[i-1][j]),
              .pe4_in  (pe4c_out[i][j-1]),
              .pw4_in  ('{1'b0, 1'b0, 1'b0, 1'b0}),
              .n_in    (sc_out[i-1][j]),
              .s_in    (nc_out[i+1][j]),
              .e_in    (e_in[i]),
              .w_in    (ec_out[i][j-1]),
              .clk_out (clk_out[i][j]),
              .clr_out (clr_out[i][j]),
              .n4_out  (n4c_out[i][j]),
              .s4_out  (s4c_out[i][j]),
              .e4_out  (e4c_out[i][j]),
              .w4_out  (w4c_out[i][j]),
              .pn4_out (pn4c_out[i][j]),
              .ps4_out (ps4c_out[i][j]),
              .pe4_out (pe4c_out[i][j]),
              .pw4_out (pw4c_out[i][j]),
              .n_out   (nc_out[i][j]),
              .s_out   (sc_out[i][j]),
              .e_out   (e_out[i]),
              .w_out   (wc_out[i][j])
            );
          end
          else begin
            block_4x4 block
            (
              .p       (p[(i*3*448)+(j*448)+447:(i*3*448)+(j*448)]),
              .g_clr   (g_clr),
              .g_clk   (g_clk),
              .clk_in  (clk_out[i+1][j]),
              .clr_in  (clr_out[i+1][j]),
              .n4_in   (n4c_out[i+1][j]),
              .s4_in   (s4c_out[i-1][j]),
              .e4_in   (e4c_out[i][j-1]),
              .w4_in   (w4c_out[i][j+1]),
              .pn4_in  (pn4c_out[i+1][j]),
              .ps4_in  (ps4c_out[i-1][j]),
              .pe4_in  (pe4c_out[i][j-1]),
              .pw4_in  (pw4c_out[i][j+1]),
              .n_in    (sc_out[i-1][j]),
              .s_in    (nc_out[i+1][j]),
              .e_in    (wc_out[i][j+1]),
              .w_in    (ec_out[i][j-1]),
              .clk_out (clk_out[i][j]),
              .clr_out (clr_out[i][j]),
              .n4_out  (n4c_out[i][j]),
              .s4_out  (s4c_out[i][j]),
              .e4_out  (e4c_out[i][j]),
              .w4_out  (w4c_out[i][j]),
              .pn4_out (pn4c_out[i][j]),
              .ps4_out (ps4c_out[i][j]),
              .pe4_out (pe4c_out[i][j]),
              .pw4_out (pw4c_out[i][j]),
              .n_out   (nc_out[i][j]),
              .s_out   (sc_out[i][j]),
              .e_out   (ec_out[i][j]),
              .w_out   (wc_out[i][j])
            );
          end
        end

        if (i==2) begin
          if (j==0) begin
            block_4x4 block
            (
              .p       (p[(i*3*448)+(j*448)+447:(i*3*448)+(j*448)]),
              .g_clr   (g_clr),
              .g_clk   (g_clk),
              .clk_in  ('{1'b0, 1'b0, 1'b0, 1'b0}),
              .clr_in  ('{1'b0, 1'b0, 1'b0, 1'b0}),
              .n4_in   ('{1'b0, 1'b0, 1'b0, 1'b0}),
              .s4_in   (s4c_out[i-1][j]),
              .e4_in   ('{1'b0, 1'b0, 1'b0, 1'b0}),
              .w4_in   (w4c_out[i][j+1]),
              .pn4_in  ('{1'b0, 1'b0, 1'b0, 1'b0}),
              .ps4_in  (ps4c_out[i-1][j]),
              .pe4_in  ('{1'b0, 1'b0, 1'b0, 1'b0}),
              .pw4_in  (pw4c_out[i][j+1]),
              .n_in    (sc_out[i-1][j]),
              .s_in    (s_in[j]),
              .e_in    (wc_out[i][j+1]),
              .w_in    (w_in[i]),
              .clk_out (clk_out[i][j]),
              .clr_out (clr_out[i][j]),
              .n4_out  (n4c_out[i][j]),
              .s4_out  (s4c_out[i][j]),
              .e4_out  (e4c_out[i][j]),
              .w4_out  (w4c_out[i][j]),
              .pn4_out (pn4c_out[i][j]),
              .ps4_out (ps4c_out[i][j]),
              .pe4_out (pe4c_out[i][j]),
              .pw4_out (pw4c_out[i][j]),
              .n_out   (nc_out[i][j]),
              .s_out   (s_out[j]),
              .e_out   (ec_out[i][j]),
              .w_out   (w_out[i])
            );
          end
          else if (j==2) begin
            block_4x4 block
            (
              .p       (p[(i*3*448)+(j*448)+447:(i*3*448)+(j*448)]),
              .g_clr   (g_clr),
              .g_clk   (g_clk),
              .clk_in  ('{1'b0, 1'b0, 1'b0, 1'b0}),
              .clr_in  ('{1'b0, 1'b0, 1'b0, 1'b0}),
              .n4_in   ('{1'b0, 1'b0, 1'b0, 1'b0}),
              .s4_in   (s4c_out[i-1][j]),
              .e4_in   (e4c_out[i][j-1]),
              .w4_in   ('{1'b0, 1'b0, 1'b0, 1'b0}),
              .pn4_in  ('{1'b0, 1'b0, 1'b0, 1'b0}),
              .ps4_in  (ps4c_out[i-1][j]),
              .pe4_in  (pe4c_out[i][j-1]),
              .pw4_in  ('{1'b0, 1'b0, 1'b0, 1'b0}),
              .n_in    (sc_out[i-1][j]),
              .s_in    (s_in[j]),
              .e_in    (e_in[i]),
              .w_in    (ec_out[i][j-1]),
              .clk_out (clk_out[i][j]),
              .clr_out (clr_out[i][j]),
              .n4_out  (n4c_out[i][j]),
              .s4_out  (s4c_out[i][j]),
              .e4_out  (e4c_out[i][j]),
              .w4_out  (w4c_out[i][j]),
              .pn4_out (pn4c_out[i][j]),
              .ps4_out (ps4c_out[i][j]),
              .pe4_out (pe4c_out[i][j]),
              .pw4_out (pw4c_out[i][j]),
              .n_out   (nc_out[i][j]),
              .s_out   (s_out[j]),
              .e_out   (e_out[j]),
              .w_out   (wc_out[i][j])
            );
          end
          else begin
            block_4x4 block
            (
              .p       (p[(i*3*448)+(j*448)+447:(i*3*448)+(j*448)]),
              .g_clr   (g_clr),
              .g_clk   (g_clk),
              .clk_in  ('{1'b0, 1'b0, 1'b0, 1'b0}),
              .clr_in  ('{1'b0, 1'b0, 1'b0, 1'b0}),
              .n4_in   ('{1'b0, 1'b0, 1'b0, 1'b0}),
              .s4_in   (s4c_out[i-1][j]),
              .e4_in   (e4c_out[i][j-1]),
              .w4_in   (w4c_out[i][j+1]),
              .pn4_in  ('{1'b0, 1'b0, 1'b0, 1'b0}),
              .ps4_in  (ps4c_out[i-1][j]),
              .pe4_in  ('{1'b0, 1'b0, 1'b0, 1'b0}),
              .pw4_in  ('{1'b0, 1'b0, 1'b0, 1'b0}),
              .n_in    (sc_out[i-1][j]),
              .s_in    (s_in[j]),
              .e_in    (wc_out[i][j+1]),
              .w_in    (ec_out[i][j-1]),
              .clk_out (clk_out[i][j]),
              .clr_out (clr_out[i][j]),
              .n4_out  (n4c_out[i][j]),
              .s4_out  (s4c_out[i][j]),
              .e4_out  (e4c_out[i][j]),
              .w4_out  (w4c_out[i][j]),
              .pn4_out (pn4c_out[i][j]),
              .ps4_out (ps4c_out[i][j]),
              .pe4_out (pe4c_out[i][j]),
              .pw4_out (pw4c_out[i][j]),
              .n_out   (nc_out[i][j]),
              .s_out   (s_out[j]),
              .e_out   (ec_out[i][j]),
              .w_out   (wc_out[i][j])
            );
          end
        end
      end
    end
  endgenerate

endmodule

`endif /* FPGA_SV */
