//========================================================================
// Cell Functional Unit
//========================================================================

`ifndef FU_SV
`define FU_SV

`include "vc/muxes.sv"

module fu
(
  (* noprune, keep = 1 *) input  logic       clk,
  (* noprune, keep = 1 *) input  logic       reset,

  (* noprune, keep = 1 *) input  logic [5:0] p,

  (* noprune, keep = 1 *) input  logic       x1,
  (* noprune, keep = 1 *) input  logic       x2,
  (* noprune, keep = 1 *) input  logic       x3,

  (* noprune, keep = 1 *) output logic       f
);

  (* noprune, keep = 1 *) logic [1:0] y2_sel;
  (* noprune, keep = 1 *) logic [1:0] y3_sel;
  (* noprune, keep = 1 *) logic       rp_sel;
  (* noprune, keep = 1 *) logic       cs_sel;

  (* noprune, keep = 1 *) logic       y2;
  (* noprune, keep = 1 *) logic       y3;
  (* noprune, keep = 1 *) logic       c;
  (* noprune, keep = 1 *) logic       c_inv;
  (* noprune, keep = 1 *) logic       d;
  (* noprune, keep = 1 *) logic       d_inv;
  (* noprune, keep = 1 *) logic       q;
  (* noprune, keep = 1 *) logic       f_inv;

  assign y2_sel = p[1:0];
  assign y3_sel = p[3:2];
  assign rp_sel = p[4];
  assign cs_sel = p[5];

  assign c = ~c_inv;
  assign d = ~d_inv;
  // TODO: double check f vs. f_inv
  assign f = f_inv;

  vc_Mux4#(1) y2_mux
  (
    .sel (y2_sel),
    .in0 (x2),
    .in1 (~x2),
    .in2 (q),
    .in3 (~q),
    .out (y2)
  );

  vc_Mux4#(1) y3_mux
  (
    .sel (y3_sel),
    .in0 (x3),
    .in1 (~x3),
    .in2 (q),
    .in3 (~q),
    .out (y3)
  );

  vc_Mux2#(1) x1_mux
  (
    .sel (x1),
    .in0 (y3),
    .in1 (y2),
    .out (c_inv)
  );

  vc_Mux2#(1) rp_mux
  (
    .sel (rp_sel),
    .in0 (c),
    .in1 (~q),
    .out (d_inv)
  );

  vc_Mux2#(1) cs_mux
  (
    .sel (cs_sel),
    .in0 (c),
    .in1 (q),
    .out (f_inv)
  );

  vc_ResetReg#(1, 0) flip_flop
  (
    .clk   (clk),
    .reset (reset),
    .q     (q),
    .d     (d)
  );

endmodule

`endif /* FU_SV */
