//========================================================================
// Simple FPGA Cell
//========================================================================

`ifndef SWITCHES_SV
`define SWITCHES_SV

`include "vc/muxes.sv"

module n_switch
(
  (* noprune, keep = 1 *) input logic [5:0] p,

  (* noprune, keep = 1 *) input logic       g_clr,
  (* noprune, keep = 1 *) input logic       g_clk,
  (* noprune, keep = 1 *) input logic       clk_in,

  (* noprune, keep = 1 *) input logic       nc_out,
  (* noprune, keep = 1 *) input logic       n4_in,
  (* noprune, keep = 1 *) input logic       ps4,

  (* noprune, keep = 1 *) output logic      clk_out,
  (* noprune, keep = 1 *) output logic      n_out,
  (* noprune, keep = 1 *) output logic      n4_out,
  (* noprune, keep = 1 *) output logic      pn4
);

  (* noprune, keep = 1 *) logic [1:0] clk_mux_sel;
  (* noprune, keep = 1 *) logic [1:0] n_out_mux_sel;
  (* noprune, keep = 1 *) logic [1:0] n4_out_mux_sel;

  assign clk_mux_sel = p[1:0];
  assign n_out_mux_sel = p[3:2];
  assign n4_out_mux_sel = p[5:4];

  assign pn4 = n4_in;

  vc_Mux4#(1) clk_mux
  (
    .sel (clk_mux_sel),
    .in0 (g_clr),
    .in1 (g_clk),
    .in2 (clk_in),
    .in3 (n_out),
    .out (clk_out)
  );

  vc_Mux3#(1) n_out_mux
  (
    .sel (n_out_mux_sel),
    .in0 (nc_out),
    .in1 (n4_in),
    .in2 (ps4),
    .out (n_out)
  );

  vc_Mux4#(1) n4_out_mux
  (
    .sel (n4_out_mux_sel),
    .in0 (nc_out),
    .in1 (n4_in),
    .in2 (ps4),
    .in3 (clk_in),
    .out (n4_out)
  );

endmodule

module s_switch
(
  (* noprune, keep = 1 *) input logic [5:0] p,

  (* noprune, keep = 1 *) input logic       g_clr,
  (* noprune, keep = 1 *) input logic       g_clk,
  (* noprune, keep = 1 *) input logic       clr_in,

  (* noprune, keep = 1 *) input logic       sc_out,
  (* noprune, keep = 1 *) input logic       s4_in,
  (* noprune, keep = 1 *) input logic       pn4,

  (* noprune, keep = 1 *) output logic      clr_out,
  (* noprune, keep = 1 *) output logic      s_out,
  (* noprune, keep = 1 *) output logic      s4_out,
  (* noprune, keep = 1 *) output logic      ps4
);

  (* noprune, keep = 1 *) logic [1:0] clr_mux_sel;
  (* noprune, keep = 1 *) logic [1:0] s_out_mux_sel;
  (* noprune, keep = 1 *) logic [1:0] s4_out_mux_sel;

  assign clr_mux_sel = p[1:0];
  assign s_out_mux_sel = p[3:2];
  assign s4_out_mux_sel = p[5:4];

  assign ps4 = s4_in;

  vc_Mux4#(1) clr_mux
  (
    .sel (clr_mux_sel),
    .in0 (g_clr),
    .in1 (g_clk),
    .in2 (clr_in),
    .in3 (s_out),
    .out (clr_out)
  );

  vc_Mux3#(1) s_out_mux
  (
    .sel (s_out_mux_sel),
    .in0 (sc_out),
    .in1 (s4_in),
    .in2 (pn4),
    .out (s_out)
  );

  vc_Mux3#(1) s4_out_mux
  (
    .sel (s4_out_mux_sel),
    .in0 (sc_out),
    .in1 (s4_in),
    .in2 (pn4),
    .out (s4_out)
  );

endmodule

module e_switch
(
  (* noprune, keep = 1 *) input logic [3:0] p,

  (* noprune, keep = 1 *) input logic       ec_out,
  (* noprune, keep = 1 *) input logic       e4_in,
  (* noprune, keep = 1 *) input logic       pw4,

  (* noprune, keep = 1 *) output logic      e_out,
  (* noprune, keep = 1 *) output logic      e4_out,
  (* noprune, keep = 1 *) output logic      pe4
);

  (* noprune, keep = 1 *) logic [1:0] e_out_mux_sel;
  (* noprune, keep = 1 *) logic [1:0] e4_out_mux_sel;

  assign e_out_mux_sel = p[1:0];
  assign e4_out_mux_sel = p[3:2];

  assign pe4 = e4_in;

  vc_Mux3#(1) e_out_mux
  (
    .sel (e_out_mux_sel),
    .in0 (ec_out),
    .in1 (e4_in),
    .in2 (pw4),
    .out (e_out)
  );

  vc_Mux3#(1) e4_out_mux
  (
    .sel (e4_out_mux_sel),
    .in0 (ec_out),
    .in1 (e4_in),
    .in2 (pw4),
    .out (e4_out)
  );

endmodule

module w_switch
(
  (* noprune, keep = 1 *) input logic [3:0] p,

  (* noprune, keep = 1 *) input logic       wc_out,
  (* noprune, keep = 1 *) input logic       w4_in,
  (* noprune, keep = 1 *) input logic       pe4,

  (* noprune, keep = 1 *) output logic      w_out,
  (* noprune, keep = 1 *) output logic      w4_out,
  (* noprune, keep = 1 *) output logic      pw4
);

  (* noprune, keep = 1 *) logic [1:0] w_out_mux_sel;
  (* noprune, keep = 1 *) logic [1:0] w4_out_mux_sel;

  assign w_out_mux_sel = p[1:0];
  assign w4_out_mux_sel = p[3:2];

  assign pw4 = w4_in;

  vc_Mux3#(1) w_out_mux
  (
    .sel (w_out_mux_sel),
    .in0 (wc_out),
    .in1 (w4_in),
    .in2 (pe4),
    .out (w_out)
  );

  vc_Mux3#(1) w4_out_mux
  (
    .sel (w4_out_mux_sel),
    .in0 (wc_out),
    .in1 (w4_in),
    .in2 (pe4),
    .out (w4_out)
  );

endmodule

`endif /* SWITCHES_SV */
